/*
 * Copyright (c) Members of the EGEE Collaboration. 2009-2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "glite_context_adaptor.hpp"

#include <saga/saga.hpp>

#include <saga/saga/adaptors/config.hpp>
#include <saga/saga/adaptors/adaptor.hpp>
#include <saga/saga/adaptors/attribute.hpp>

#include "proxy_funcs.hpp"

using namespace glite_context_adaptor;

#define GLITE_CONTEXT_ADAPTOR_TYPE "glite"

SAGA_ADAPTOR_REGISTER(context_adaptor);

///////////////////////////////////////////////////////////////////////////////
//  constructor
context_cpi_impl::context_cpi_impl(proxy* p, 
                                   cpi_info const & info,
                                   saga::ini::ini const & glob_ini, 
                                   saga::ini::ini const & adap_ini,
                                   TR1::shared_ptr<saga::adaptor> adaptor)
   : base_cpi(p, info, adaptor, cpi::Noflags)
{
   saga::adaptors::attribute attr(this);

   if ( attr.attribute_exists(saga::attributes::context_type) )
   {
      if ( GLITE_CONTEXT_ADAPTOR_TYPE != 
         attr.get_attribute(saga::attributes::context_type) )
      {
         std::ostringstream err;
         err << "Can't handle context types others than "
             << GLITE_CONTEXT_ADAPTOR_TYPE
             << " (got "
             << attr.get_attribute(saga::attributes::context_type)
             << ")";

         SAGA_ADAPTOR_THROW(err.str(), saga::BadParameter);
      }
   }
}

context_cpi_impl::~context_cpi_impl (void)
{
}

void context_cpi_impl::sync_set_defaults (saga::impl::void_t &)
{    
   saga::adaptors::attribute attr (this);

   if ( attr.attribute_exists (saga::attributes::context_type) )
   {
      if ( GLITE_CONTEXT_ADAPTOR_TYPE != 
           attr.get_attribute (saga::attributes::context_type) )
      {
         std::ostringstream err;
         err << "Can't handle context types others than "
             << GLITE_CONTEXT_ADAPTOR_TYPE
             << " (got "
             << attr.get_attribute(saga::attributes::context_type)
             << ")";

         SAGA_ADAPTOR_THROW(err.str(), saga::BadParameter);
      }

      if ( "" == attr.get_attribute (saga::attributes::context_type) )
      {
         SAGA_ADAPTOR_THROW ("Context Type cannot have an empty value", 
                             saga::IncorrectState);
      }

      if ( !attr.attribute_exists(saga::attributes::context_userproxy) )
      {
         attr.set_attribute(saga::attributes::context_userproxy,
                            GetProxyPath());
      }
   }
}


saga::impl::adaptor_selector::adaptor_info_list_type
context_adaptor::adaptor_register (saga::impl::session * s)
{
   // list of implemented cpi's
   saga::impl::adaptor_selector::adaptor_info_list_type infos;
   preference_type prefs; 

   context_cpi_impl::register_cpi(infos, prefs, adaptor_uuid_);

   // create a default security context if this is a default session
   if ( s->is_default_session() )
   {
      std::vector<std::pair <std::string, std::string> > entries;

      std::pair<std::string, std::string> entry(saga::attributes::context_type, 
                                                "glite");
       entries.push_back (entry);

       s->add_proto_context (entries);
   }

   return infos;
}

